/**@file

This library contains functions related to WinCC OA Managers

@par Creation Date
        18/02/2015


@author Piotr Golonka (CERN EN/ICE)
*/


#uses "fwConfigs/fwConfigs.ctl"
#uses "fwConfigs/fwDPELock.ctl"

/** Returns the list of Ctrl scripts executed by a given Ctrl Manager.

    @par systemName (input) the name of the system to be queried
    @par ctrlManNum (input) the manager number to be queried
    @par exceptionInfo (output) the standard exception-handling variable

    @returns the list of names of scripts executed by given Ctrl manager, or empty in case of error.
*/
synchronized dyn_string fwManager_getScriptsRunByCtrlMan(string systemName, unsigned ctrlManNum, dyn_string &exceptionInfo)
{

  dyn_string listOfScripts;

  string ctrlDbgDP =  systemName+"_CtrlDebug_CTRL_"+ctrlManNum;  

  if(!dpExists(ctrlDbgDP)) {
    fwException_raise(exceptionInfo,"ERROR","Cannot retrieve list of scripts for Ctrl "+
                      ctrlManNum+". DP does not exist "+ctrlDbgDP,"");
     return listOfScripts; 
  }


    // lock, then trigger the command on the Debug datapoint

    fwDPELock_tryLock (ctrlDbgDP+".Command", exceptionInfo, 1, 1);
    if (dynlen(exceptionInfo)) return listOfScripts;

    dpSetWait(ctrlDbgDP+".Command:_original.._value", "info scripts");

    time tRequest, tResponse;
    dyn_string result;

    // to ensure that the response is the one to our request, we check the timestamps
    dpGet(ctrlDbgDP+".Command:_online.._stime", tRequest);

    int count=0;
    bool hasResult=false;
    while (!hasResult) {
       delay(0,50);
       count++;
       dpGet(ctrlDbgDP+".Result:_online.._stime", tResponse,
             ctrlDbgDP+".Result:_online.._value", result);  
       if (tResponse >= tRequest) { hasResult=true; break;};
       if (count >= 20) break;
    }


    if (!hasResult) {
	// note: sometimes the manager is not running (even though indicated in _Connections DP.
	// in this case, the CtrlDebug does not respond. Report it in the log, yet do not fail.
	DebugTN("WARNING: fwManager_getScriptsRunByCtrlMan: CTRL Manager "+(int)ctrlManNum + " does not respond");
    }

    // the result is a list of strings in form such as
    // ScriptId: 0;unSelectDeselect.ctl
    // ScriptId: 1;unDistributedControl.ctl
    // ScriptId: 2; current thread: 2;unSystemIntegrity.ctl
    // ScriptId: 3;unSendMail.ctl
    // ScriptId: 4;scheduler.ctc
    // ScriptId: 5;unDeviceSet.ctl
    for (int i=1;i<=dynlen(result);i++) {
      dyn_string ds=strsplit(result[i],";");
      if (dynlen(ds)>1) {
        // we take the last token (see example of scripId:2 above!
        string scriptName=ds[dynlen(ds)];
        dynAppend(listOfScripts, scriptName);
      }
    }

    // unlock the Debug datapoint
    fwDPELock_unlock(ctrlDbgDP+".Command", exceptionInfo);


   return listOfScripts;
}
